/*  fgwirc - fungw based minimal irc client
    Copyright (C) 2019  Tibor 'Igor2' Palinkas

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

    Project page and contact: http://repo.hu/projects/fgwirc
*/

#include <ctype.h>
#include <libfungw/fungw.h>
#include <sockev/sockev.h>
#include "action.h"
#include "irc.h"
#include "channel.h"
#include "window.h"

int channel_join(win_t *win, const char *chan, const char *key)
{
	char tmp[520];
	sockev_t sock;
	int new_sid = win->target.sid;

	if (strlen(chan) > 128) {
		ACT_ERROR("join", "Channel name too long");
		return FGW_ERR_ARG_CONV;
	}

	/* set up win->sid */
	ACT_SET_SERVER_AFFINITY(win, chan, new_sid);
	sock = irc_server_sock(new_sid);

	win_set_target(win, TARGET_CHANNEL, new_sid, chan, key);

	free(win->name);
	if (sock != SOCKEV_INVALID)
		sprintf(tmp, "(would join %s)", chan);
	else
		sprintf(tmp, "(joining %s)", chan);
	win->name = fgw_strdup(tmp);
	win_set_target_state(win, TGTST_INPROG);
	if (sock != SOCKEV_INVALID)
		sockev_printf(sock, "join %s %s\r\n", chan, key);
	win->dirty = 1;

	return 0;
}

int channel_part(int sid, const char *chan, const char *msg)
{
#warning TODO: this does not handle comma list
	win_t *w = win_target2win(TARGET_CHANNEL, sid, chan);
	sockev_t sock = irc_server_sock(sid);

	if (w != NULL) {
		win_set_target(w, TARGET_NONE, sid, win_target_keep, win_target_keep);
		w->dirty = 1;
	}

	if (sock != SOCKEV_INVALID) {
		sockev_printf(sock, "part %s :%s\r\n", chan, msg);
		if (w != NULL) {
			char tmp[520];
			free(w->name);
			sprintf(tmp, "(parted %s)", chan);
			w->name = fgw_strdup(tmp);
		}
	}

	return 0;
}


int channel_join_pending_by_sid(sockev_t sock, int sid)
{
	int n;
	for(n = 0; n < windows.used; n++) {
		win_t *w = windows.array[n];
		if ((w->target.type != TARGET_CHANNEL) || (w->target.sid != sid)) continue;
		sockev_printf(sock, "join %s %s\r\n", w->target.name, w->target.key);
	}
	return 0;
}


int channel_disconnected_by_sid(int sid)
{
	char tmp[256];
	int n;

	for(n = 0; n < windows.used; n++) {
		win_t *w = windows.array[n];
		if ((w->target.type != TARGET_CHANNEL) || (w->target.sid != sid)) continue;
		free(w->name);
		sprintf(tmp, "(would join %s)", w->target.name);
		w->name = fgw_strdup(tmp);
		w->dirty = 1;
		win_set_target_state(w, TGTST_INPROG);
	}
	return 0;
}

