/*  fgwirc - fungw based minimal irc client
    Copyright (C) 2019  Tibor 'Igor2' Palinkas

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

    Project page and contact: http://repo.hu/projects/fgwirc
*/

#ifndef TARGET_H
#define TARGET_H

#include "global_typedefs.h"

typedef enum target_type_e {
	TARGET_NONE = 0,
	TARGET_CHANNEL,
	TARGET_NICK,
	TARGET_SERVER_STATUS,
	TARGET_FGWIRC_STATUS
} target_type_t;

#define target_type_is_irc(t) \
	(((t) == TARGET_CHANNEL) || ((t) == TARGET_NICK) || ((t) == TARGET_SERVER_STATUS))

typedef enum target_state_e {
	TGTST_NORMAL = 0,   /* e.g. channel joined */
	TGTST_INPROG = 1    /* command sent, waiting for the result (e.g. channel joining) */
} target_state_t;

struct target_s {
	target_type_t type;
	int sid;            /* server ID */
	char *name;         /* channel name or query nick */

	/* not part of hash calculation: */
	target_state_t state;
	char *key;          /* channel key */
};


#define HT_HAS_CONST_KEY
typedef target_t *httw_key_t;
typedef const target_t *httw_const_key_t;
typedef struct win_s *httw_value_t;
#define HT(x) httw_ ## x
#include "genht/ht.h"
#undef HT

int httw_keyeq(httw_const_key_t a, httw_const_key_t b);
unsigned httw_keyhash(httw_const_key_t k);

#endif

