#include <stdlib.h>
#include "sockev.h"
#include "sockev_internal.h"
#include "filters.h"

void sockev_filter_next(sockev_t id, sockev_f_dir_t event, void *data, int len, int cookie)
{
	switch(event) {
		case SOCKEV_F_IN:
		case SOCKEV_F_CLOSE:
			cookie--;
			if (cookie < 0)
				sockev_filter_final_in(id, NULL, SOCKEV_F_IN, data, len, 0);
			else
				SOCKS[id].filt_cb[cookie](id, &(SOCKS[id].filt_data[cookie]), event, data, len, cookie);
			break;
		case SOCKEV_F_OUT:
			cookie++;
			if (cookie >= SOCKS[id].filters)
				sockev_filter_final_out(id, NULL, SOCKEV_F_OUT, data, len, 0);
			else
				SOCKS[id].filt_cb[cookie](id, &(SOCKS[id].filt_data[cookie]), event, data, len, cookie);
			break;
		default: return;
	}
}

void sockev_filter_final_out(sockev_t id, void **sockdata, sockev_f_dir_t event, const void *data, int len, int cookie)
{
	if (event == SOCKEV_F_OUT) {
		/* configurable final may take over */
		if (SOCKS[id].final_out != NULL) {
			SOCKS[id].final_out(id, sockdata, event, data, len, cookie);
			return;
		}

#warning TODO: attempt to write some first (from the buffer, if not empty)
		sockev_buffer_append(&SOCKS[id].wbuff, data, len);
		sockev_setpoll(id);
	}
}

void sockev_filter_final_in(sockev_t id, void **sockdata, sockev_f_dir_t event, void *data, int len, int cookie)
{
	if (event == SOCKEV_F_IN) {
		char *buff = data;

		/* configurable final may take over */
		if (SOCKS[id].final_in != NULL) {
			SOCKS[id].final_in(id, sockdata, event, data, len, cookie);
			return;
		}

		if (SOCKS[id].hooks.read != NULL) {
			buff[len] = '\0';
			SOCKS[id].hooks.read(id, buff, len);
		}
	}
}

sockev_t sockev_cfg_filters(sockev_t id, const sockev_filters_t *filters)
{
	const sockev_filters_t *f;
	int n;

	if (wrong_id(id, -1))
		return -1;

	SOCKS[id].filters = 0;

	if (filters == NULL)
		return id;

	/* count filters */
	for(f = filters; f->filter != NULL; f++)
		SOCKS[id].filters++;

#warning consider reconfiguration - if SOCKS[id].filt_cb != NULL, do something; may be invalid and return error?
	/* allocate filter arrays */
	SOCKS[id].filt_cb   = malloc(sizeof(sockev_filter_cb) * SOCKS[id].filters);
	SOCKS[id].filt_data = calloc(sizeof(void *), SOCKS[id].filters);

	for(n = 0, f = filters; f->filter != NULL; n++,f++) {
		SOCKS[id].filt_cb[n] = f->filter;
		f->filter(id, &(SOCKS[id].filt_data[n]), SOCKEV_F_INIT, f->filter_init_data, 0, 0);
	}
	return id;
}

void sockev_del_filters(sockev_t id)
{
	if (SOCKS[id].filt_cb != NULL) {
		free(SOCKS[id].filt_cb);
		SOCKS[id].filt_cb = NULL;
	}
	if (SOCKS[id].filt_data != NULL) {
		free(SOCKS[id].filt_data);
		SOCKS[id].filt_data = NULL;
	}
}

void *sockev_find_filter(sockev_t id, sockev_filter_cb filter, int no)
{
	int n;
	for(n = 0; n < SOCKS[id].filters; n++) {
		if (SOCKS[id].filt_cb[n] == filter) {
			if (no == 0)
				return SOCKS[id].filt_data[n];
			no--;
		}
	}
	return NULL;
}
