/* low level test for packet filter; tests corner cases for strange reads */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "../sockev.h"
#include "../filters.h"
#include "../filt_packet.h"

void *sockdata;
P_int32_t maxlen = 128;

void sockev_shutdown(sockev_t id, int direction)
{
	printf("Shutdown: %d %d\n", id, direction);
}

char *vis(const char *in, int len)
{
	static char out[1024];
	const char *i;
	char *o;
	for(i = in, o = out; len > 0; len--, o++, i++) {
		if (*i == '\\') {
			*o = '\\';
			o++;
			*o = '\\';
		}
		else if ((*i >= 32) && (*i < 127)) {
			*o = *i;
		}
		else {
			int l;
			l = sprintf(o, "\\%x", *i);
			o += l-1;
		}
	}
	*o = '\0';
	return out;
}

int pack(char *out, char *in)
{
	int l;
	l = strlen(in);
	*(P_int32_t *)out = htonl(l);
	memcpy(out+4, in, l);
	return l+4;
}

void sockev_filter_next(sockev_t id, sockev_f_dir_t event, void *data, int len, int cookie)
{
	printf(" filter_next ");
	switch(event) {
		case SOCKEV_F_INIT:
			printf("init\n");
			break;
		case SOCKEV_F_IN:
			printf("IN %d/'%s'\n", len, vis((char *)data, len));
			break;
		case SOCKEV_F_OUT:
			printf("OUT %d/'%s'\n", len, vis((char *)data, len));
			break;
		case SOCKEV_F_OPEN:
			printf("open\n");
			break;
		case SOCKEV_F_CLOSE:
			printf("close\n");
			break;
	}
}

int main()
{
	char s[1024];
	int len;

	/* write test */
	printf("*** Write\n");
	sockev_filter_packet(42, &sockdata, SOCKEV_F_INIT, &maxlen, sizeof(maxlen), 314);
	sockev_filter_packet(42, &sockdata, SOCKEV_F_OUT, "foobar", 6, 314);
	sockev_filter_packet(42, &sockdata, SOCKEV_F_OUT, "foobar", maxlen+1, 314);

	/* read test */
	printf("\n\n*** Read\n");
	sockev_filter_packet(42, &sockdata, SOCKEV_F_INIT, &maxlen, sizeof(maxlen), 314);
	sockev_filter_packet(42, &sockdata, SOCKEV_F_OPEN, NULL, 0, 314);

	printf("\natomic\n");
	len = pack(s, "foobar");
	sockev_filter_packet(42, &sockdata, SOCKEV_F_IN, s, len, 314);

	printf("\natomic len and atomic data\n");
	sockev_filter_packet(42, &sockdata, SOCKEV_F_IN, s, 4, 314);
	sockev_filter_packet(42, &sockdata, SOCKEV_F_IN, s+4, 6, 314);

	printf("\natomic len and split data\n");
	sockev_filter_packet(42, &sockdata, SOCKEV_F_IN, s, 4, 314);
	sockev_filter_packet(42, &sockdata, SOCKEV_F_IN, s+4, 2, 314);
	sockev_filter_packet(42, &sockdata, SOCKEV_F_IN, s+6, 4, 314);

	printf("\nsplit len and split data\n");
	sockev_filter_packet(42, &sockdata, SOCKEV_F_IN, s, 2, 314);
	sockev_filter_packet(42, &sockdata, SOCKEV_F_IN, s+2, 2, 314);
	sockev_filter_packet(42, &sockdata, SOCKEV_F_IN, s+4, 2, 314);
	sockev_filter_packet(42, &sockdata, SOCKEV_F_IN, s+6, 4, 314);

	printf("\nsplit len mixed with split data\n");
	sockev_filter_packet(42, &sockdata, SOCKEV_F_IN, s, 1, 314);
	sockev_filter_packet(42, &sockdata, SOCKEV_F_IN, s+1, 4, 314);
	sockev_filter_packet(42, &sockdata, SOCKEV_F_IN, s+5, 2, 314);
	sockev_filter_packet(42, &sockdata, SOCKEV_F_IN, s+7, 3, 314);

	return 0;
}
